/**
 *  OsdReader plugin for the Video Disk Recorder 
 *
 *  status.c  -  status monitor class
 *
 *  (c) 2010 Jean-Christophe Duberga < jc AT duberga.net >
 *
 * This code is distributed under the terms and conditions of the
 * GNU GENERAL PUBLIC LICENSE. See the file COPYING for details.
 *
 */

#include <unistd.h>
#include "status.h"
#include "setup.h"
#include <espeak/speak_lib.h>
#include <pulse/simple.h>
#include <pulse/error.h>
#include <pulse/gccmacro.h>

#define BUFSIZE 1024

static pa_sample_spec ss;
pa_simple *s = NULL;
const char *stream_name = "vdr-osdreader";
int ret = 1;


static int SynthCallback(short *wav, int numsamples, espeak_EVENT *events)
{	
   ssize_t r = 0;
   int error = 0;
   
   esyslog("OsdReader plugin: callback received %d samples", numsamples);

   if( wav == NULL)
     {
	if (pa_simple_drain(s, &error) < 0) 
	  {	     
	     esyslog("OsdReader plugin: pa_simple_drain() failed: %s", pa_strerror(error));
	  }
		
	return(0);
     }
  
   if ( s )
     {
	if ( pa_simple_write(s, wav+r, (size_t) numsamples*2, &error) <  0 )
	  {
	     esyslog("OsdReader plugin: pa_simple_write() failed: %s\n", pa_strerror(error));
	  }
	
     }	

   return 0;
}

cStatusOsdReader::cStatusOsdReader()
{
   data_path = NULL;
   synth_flags = espeakCHARS_AUTO | espeakPHONEMES | espeakENDPAUSE;
   samplerate = espeak_Initialize(AUDIO_OUTPUT_SYNCHRONOUS,0,data_path,0);   
   espeak_SetSynthCallback(SynthCallback);
   char *voicename = OsdReaderSetup.voice;
   
   esyslog("OsdReader plugin: espeak initialized with %d Hz samplerate", samplerate);
   
    if ( espeak_SetVoiceByName(voicename) != EE_OK)
     {
	esyslog("OsdReader plugin: failed to set voice %s", voicename);
     }
   
   ss.format = PA_SAMPLE_S16LE;
   ss.rate = samplerate;
   ss.channels = 1;
   
   int error = 0;
   
   /* Create a new playback stream */
   if ( ! (s = pa_simple_new(NULL, "vdr-osdreader", PA_STREAM_PLAYBACK, NULL, "playback", &ss, NULL, NULL, &error))) 
     {	
	esyslog("OsdReader plugin: pa_simple_new() failed: %s", pa_strerror(error));
     }
   
   Start();
}

cStatusOsdReader::~cStatusOsdReader()
{
   if (s)
     pa_simple_free(s);
}

void cStatusOsdReader::Action()
{
   while ( Running() )
     {
	if ( ! messages.empty() )
	  {
	     std::string message = messages.front();
	     messages.pop();
	     
	     espeak_Synth(message.c_str(),strlen(message.c_str()),0,POS_CHARACTER,0,synth_flags,NULL,NULL);
	     espeak_Synchronize();
	     
	  }
	else
	  {
	     // remove that and use a condwait instead
	     esyslog("OsdReader plugin: sleeping 1 second"); 
	     sleep(1);
	  }
     }
   
}

void cStatusOsdReader::say(const char * Message)
{
   messages.push( Message);
}


void cStatusOsdReader::OsdStatusMessage(const char * Message)
{
  if (Message && OsdReaderSetup.active)
  {
     say(Message);
  }
}


void cStatusOsdReader::ChannelSwitch(const cDevice * Device, int ChannelNumber)
{   
   // esyslog("OsdReader plugin: cStatusOsdReader::ChannelSwitch  %d %d", Device->CardIndex(), ChannelNumber);
   
   if (OsdReaderSetup.active)
     {
//	esyslog("OsdReader plugin: cStatusOsdReader::ChannelSwitch  %d %d", Device->CardIndex(), ChannelNumber);
     }
}


void cStatusOsdReader::Recording(const cDevice *Device, const char *Name, const char *FileName, bool On)
{
  if (OsdReaderSetup.active)
  {
//    dsyslog("OsdReader plugin: cStatusOsdReader::Recording  %d %s", Device->CardIndex(), Name);
  }
}


void cStatusOsdReader::Replaying(const cControl *Control, const char *Name, const char *FileName, bool On)
{
  if (OsdReaderSetup.active)
  {
//    dsyslog("OsdReader plugin: cStatusOsdReader::Replaying  %s", Name);
  }
}


void cStatusOsdReader::SetVolume(int Volume, bool Absolute)
{
  if (OsdReaderSetup.active)
  {
//    dsyslog("OsdReader plugin: cStatusOsdReader::SetVolume  %d %d", Volume, Absolute);
  }
}


void cStatusOsdReader::OsdClear()
{
  if (OsdReaderSetup.active)
  {
//    dsyslog("OsdReader plugin: cStatusOsdReader::OsdClear");
  }
}


void cStatusOsdReader::OsdTitle(const char * Title)
{
  if (OsdReaderSetup.active)
  {
    dsyslog("OsdReader plugin: cStatusOsdReader::OsdTitle '%s'", Title);
  }
}




void cStatusOsdReader::OsdHelpKeys(const char * Red, const char * Green, const char * Yellow, const char * Blue)
{
  if (OsdReaderSetup.active)
  {
//    dsyslog("OsdReader plugin: cStatusOsdReader::OsdHelpKeys %s - %s - %s - %s", Red, Green, Yellow, Blue);
  }
}


void cStatusOsdReader::OsdItem(const char * Text, int Index)
{
  if (OsdReaderSetup.active)
  {
    dsyslog("OsdReader plugin: cStatusOsdReader::OsdItem %s, %d", Text, Index);
  }
}


void cStatusOsdReader::OsdCurrentItem(const char * Text)
{
  if (OsdReaderSetup.active)
  {
    dsyslog("OsdReader plugin: cStatusOsdReader::OsdCurrentItem %s", Text);
    say(Text);     
  }
}


void cStatusOsdReader::OsdTextItem(const char * Text, bool Scroll)
{
  if (OsdReaderSetup.active)
  {
     dsyslog("OsdReader plugin: cStatusOsdReader::OsdTextItem %s %d", Text, Scroll);
  }
}


void cStatusOsdReader::OsdChannel(const char * Text)
{
  if (OsdReaderSetup.active)
  {
     dsyslog("OsdReader plugin: cStatusOsdReader::OsdChannel >%s<", Text);
     say(Text);
  }
}


void cStatusOsdReader::OsdProgramme(time_t PresentTime, const char * PresentTitle, const char * PresentSubtitle, time_t FollowingTime, const char * FollowingTitle, const char * FollowingSubtitle)
{
  if (OsdReaderSetup.active)
  {
//    dsyslog("OsdReader plugin: cStatusOsdReader::OsdProgramme PT  : %s", PresentTitle);
//    dsyslog("OsdReader plugin: cStatusOsdReader::OsdProgramme PST : %s", PresentSubtitle);
//    dsyslog("OsdReader plugin: cStatusOsdReader::OsdProgramme FT  : %s", FollowingTitle);
//    dsyslog("OsdReader plugin: cStatusOsdReader::OsdProgramme FST : %s", FollowingSubtitle);
  }
}
